package services.ejb;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import services.ejb.InventoryManager;

public @Stateless class InventoryManagerBean implements InventoryManager {

	private static Log log = LogFactory.getLog(InventoryManagerBean.class);

	@Resource(mappedName = "java:JmsXA")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = "queue/replenish")
	private Queue replenishmentQueue;

	public void allocateInventory(long productId
			, long quantitySold
			, long distributionCenterId) {

		this.replenishInventory(1, 1, 10);

	}

	@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
	private void replenishInventory(long productId
			, long distributionCenterId
			, long backOrderQuantity) {

		Connection connection           = null;
		Session session                 = null;
		MessageProducer messageProducer = null;
		MapMessage whatToReplenish      = null;

		try {
			connection      = connectionFactory.createConnection();
			session         = connection.createSession(true, 0);
			messageProducer = session.createProducer(replenishmentQueue);
			whatToReplenish = session.createMapMessage();

			whatToReplenish.setLong("productId", productId);
			whatToReplenish.setLong("distributionCenterId", distributionCenterId);
			whatToReplenish.setLong("backOrderQuantity", backOrderQuantity);

			messageProducer.send(whatToReplenish);

			connection.close();
		} catch (JMSException jmsError) {
			log.error("Failed to send a replenishment message through JMS!", jmsError);
		}

	}

}
